/*********************************************************************
*               SEGGER MICROCONTROLLER SYSTEME GmbH                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2007  SEGGER Microcontroller Systeme GmbH         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS version: 3.52d                                         *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c (for Freescale Coldfire and IAR)
Purpose : LED interface for M52223EVB evaluation board
--------  END-OF-HEADER  ---------------------------------------------
*/

#define BSP_C
#include "BSP.h"


/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/****** Assign LEDs to Ports ****************************************/
#define _LED0_BIT         0
#define _LED1_BIT         1
#define _LEDPORT_STATE    (*(volatile unsigned char*)0x4010000F)
#define _LEDPORT_LATCH    (*(volatile unsigned char*)0x40100027)
#define _LED_LATCH        (*(volatile unsigned char*)0x4010000F)

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       LED_Init()
*/
void BSP_Init(void) {
  _LEDPORT_LATCH |= (1 << _LED0_BIT) | (1 << _LED1_BIT);
}

/*********************************************************************
*
*       LED switching routines
*       LEDs are switched on with high level on port lines
*/
void BSP_SetLED(int Index) {
  _LED_LATCH |=	 (1 << Index);             /* Switch on LED 0     */
}

void BSP_ClrLED(int Index) {
  _LED_LATCH &= ~(1 << Index);            /* Switch off LED 0    */
}

void BSP_ToggleLED(int Index) {
  if (_LEDPORT_STATE & (1 << Index)) {    /* LED is switched off */
    BSP_ClrLED(Index);
  } else {
    BSP_SetLED(Index);
  }
}

/****** EOF *********************************************************/

