/*********************************************************************
*               SEGGER MICROCONTROLLER SYSTEME GmbH                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (C) 2006   SEGGER Microcontroller Systeme GmbH               *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS version: 3.32j                                         *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : Interrupts.asm
Purpose : Interrupt and exception vector table
--------  END-OF-HEADER  ---------------------------------------------
*/

/*********************************************************************
*
*       Exception vector table
*
**********************************************************************
*/
         .global VECTOR_TABLE
         .global _VECTOR_TABLE
         .global start

         .extern __SRAM
         .extern __FLASH
         .extern __SP_INIT
         .extern _main
         .extern _OS_SegmentInit_CodeWarrior

         
         .text

/*
 * Exception Vector Table
 */
         VECTOR_TABLE:
         _VECTOR_TABLE:
        .long      __SP_INIT                     // Initial SP          
        .long      start                         // Initial PC          

        .org 4 * 0x2    /* Offset in vector table */
        .long      ExceptionHandler_AccessError
        .long      ExceptionHandler_AddressError
        .long      ExceptionHandler_IllegalInstruction
        .long      ExceptionHandler_DivideByZero
        .org 4 * 0x8    /* Offset in vector table */
        .long      ExceptionHandler_Privilege


        .org 4 * 32    // Traps (software interrupts)
        .long UndefinedException                  // TrapHandler0
        .long UndefinedException                  // TrapHandler1
        .long UndefinedException                  // TrapHandler2
        .long UndefinedException                  // TrapHandler3
        .long UndefinedException                  // TrapHandler4
        .long UndefinedException                  // TrapHandler5
        .long UndefinedException                  // TrapHandler6
        .long UndefinedException                  // TrapHandler7
        .long UndefinedException                  // TrapHandler8
        .long UndefinedException                  // TrapHandler9
        .long UndefinedException                  // TrapHandler10
        .long UndefinedException                  // TrapHandler11
        .long UndefinedException                  // TrapHandler12
        .long UndefinedException                  // TrapHandler13
        .long UndefinedException                  // TrapHandler14
        .long UndefinedException                  // TrapHandler15

        .org 4 * 64
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler
        .long ISR_Handler

/*********************************************************************
*
*       Exception handlers
*
**********************************************************************
*/
ExceptionHandler_AccessError:         bra.b ($)
ExceptionHandler_AddressError:        bra.b ($)
ExceptionHandler_IllegalInstruction:  bra.b ($)
ExceptionHandler_DivideByZero:        bra.b ($)
ExceptionHandler_Privilege:           bra.b ($)

/*********************************************************************
*
*       Trap handlers
*
**********************************************************************
*/
UndefinedException: 
         move.w    #0x2700, sr                   // disable interrupts
         bra.b ($)                               // branch to pc 

/*********************************************************************
*
*       Interrupt handlers
*
**********************************************************************
*/
         .extern _OS_irq_handler

ISR_Handler:
         move.w    #0x2500,sr                    // Disable low-prio interrupts. This needs to be the first instruction since interrupts are disabled for just one instruction after start of exception.
         lea.l     (-20,A7), a7                  // Create stack frame
         movem.l   d0-d2/a0-a1, (a7)             // Save temp registers
         move.l    (0x14,a7), d0                 // Load SR 
         lea.l     (-4,A7), a7                   // Create stack frame 
         move.l    d0,(a7)                       // Push on stack (Codewarrior's calling convention differs from IAR)
         jsr       (_OS_irq_handler)             // Call actual "C" handler
         lea.l     (4,A7), a7                    // cleanup stack
         movem.l   (a7), d0-d2/a0-a1             // Restore temp registers
         lea.l     (20,a7), a7                   // cleanup stack
         rte
   
         .text
         .org 0x400
start:
         move.l    #__SRAM,d0                    // Initialize RAM base address 
         add.l     #0x21,d0
         movec     d0,RAMBAR1
         move.l    #0,d0                         // Initialize ROM base address 
         add.l     #0x61,d0
         movec     d0,RAMBAR0
         move.l    #__SP_INIT, sp                // Locate Stack Pointer 
         move.l    #__FLASH, d0                  // initialize vbr for flash target          
         movec.l   d0, vbr
         jsr       _OS_SegmentInit_CodeWarrior   // Common startup code 
         jsr       _main                         // Jump to the main process 

         bra        .
         nop
         nop
         halt

         .end

/*****  EOF  ********************************************************/

